#!/bin/sh
# Install postgresql-9.1
#
#
# Some checkings before installing

trap "on_error; on_exit; exit" INT QUIT TERM

source /usr/local/bin/functions.sh
source ./application_checks.sh
source ./packages_template.sh
source ./user.sh
source ./web.sh
source ./postgresql.sh

app="innovaphone-reporting"

installed_packages=""
cfg_tmp_dir=""
config_pg_dir=""
config_pg_file=""
innovaphone_user=""
innovaphone_group=""
database_user=""
database_name=""
cron_table=""
replication_script=""
replication_out=""
ldap_conf=""

function on_exit() {
    if [ -n "$cfg_tmp_dir" ]; then
        rm -rf $cfg_tmp_dir
    fi
    apt-get clean
} 

function on_error() {
    # cron
    if [ -n "$cron_table" ]; then
	if [ -f "$cron_table" ]; then
            rm $cron_table
	fi
    fi
    # drop database
    if [ -n "$database_name" ]; then
        sudo -u postgres dropdb $database_name
    fi
    if [ -n "$replication_script" -a -f "$replication_script" ]; then
        rm $replication_script
    fi
    if [ -n "$replication_out" -a -f "$replication_out" ]; then
        rm $replication_out
    fi
    # drop database user
    if [ -n "$database_user" ]; then
        sudo -u postgres psql -c "DROP USER \"${database_user}\"" postgres 2>/dev/null
    fi
    # uninstall packages
    if [ -n "$installed_packages" ]; then
        apt-get -y --force-yes purge $installed_packages 
	    apt-get autoremove
    fi
    # linux-user
    if [ -n "$innovaphone_user" ]; then
        deluser $innovaphone_user 2>/dev/null
    fi
    if [ -n "$innovaphone_group" ]; then
        delgroup $innovaphone_group 2>/dev/null
    fi
    if [ -d /home/${app} ]; then
        rm -rf /home/${app}
    fi
    if [ -n "$cfg_tmp_dir" -a -n "$config_pg_dir" ]; then
        cp $cfg_tmp_dir/postgresql.conf $config_pg_dir 2>/dev/null
        cp $cfg_tmp_dir/pg_hba.conf $config_pg_dir 2>/dev/null
    fi
    if [ -n "$ldap_conf" ]; then
        cp $cfg_tmp_dir/ldap.conf $ldap_conf
    fi
    on_exit
}

# check application, hotfix, version etc.
check_application ${app}

if [ ! -d /tmp ]; then
    log_msg "Directory tmp does not exists, create"
    mkdir /tmp
    chmod 777 /tmp
    chmod a+t /tmp    
fi

# Keep the config files that will be modified in case something goes wrong...
cfg_tmp_dir=`mktemp -d /tmp/config.$$.XXXX`
if [ $? -ne 0 ]; then
    log_error "Can't create temp config file, exiting..."
    on_error
    exit 1
fi
chmod 777 $cfg_tmp_dir

install_packages install "php5-gd php5-ldap"
if [ $? -ne 0 ]; then
    exit 1
fi

# limit the returned entries and search time
if [ -f "/etc/ldap/ldap.conf" ]; then
    ldap_temp=`mktemp /tmp/ldap.XXXXXX`
    ldap_conf="/etc/ldap/ldap.conf"
    if [ $? -eq 0 ]; then
        sed -e "/SIZELIMIT/ d" -e "/TIMELIMIT/ d" -e "/TLS_REQCERT/ d" < $ldap_conf > $ldap_temp
        echo "SIZELIMIT 10" >> $ldap_temp
        echo "TIMELIMIT 5" >> $ldap_temp
        echo "TLS_REQCERT never" >> $ldap_temp
        cp $ldap_conf $cfg_tmp_dir
        mv $ldap_temp $ldap_conf
        chmod +r $ldap_conf
    fi
fi

# call create_user function of user.sh
create_user ${app}

# call postgresql function of postgresql.sh
postgresql ${app} $cfg_tmp_dir reporting_database.psql reporting

reporting_conf=/home/${app}/reporting.conf
touch $reporting_conf
echo '<?xml version="1.0" encoding="utf-8"?>' > $reporting_conf
echo '<config>' >> $reporting_conf
echo '</config>' >> $reporting_conf
chmod 770 $reporting_conf
chown ${app}:${app} $reporting_conf

log_msg "Install reporting files"

install_web ${app} "web/${app}"

# Install reporting cleanup cron job
cron_table="/etc/cron.d/reporting_cleanup"
script_folder="/usr/local/bin/"
cleanup_script="reporting_cleanup.sh"
cleanup_php="/var/www/innovaphone/apps/${app}/cdr_cleanup.php"
cleanup_log="/var/www/innovaphone/apps/${app}/log/reporting_cleanup.log"
if [ -f $cleanup_script ]; then
    chown ${app}:www-data ${cleanup_script}
    chmod 700 ${cleanup_script}
    mv ${cleanup_script} ${script_folder}
else
    log_error "${cleanup_script} not available"
    on_error
    exit 1
fi
if grep $cleanup_script < $cron_table 1>/dev/null
then
    echo "$(date '+%Y-%m-%d %H:%M:%S') - reporting cleanup script already in crontab"
else
    echo "$(date '+%Y-%m-%d %H:%M:%S') - inserting reporting cleanup script to crontab"
    echo "PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin" >> ${cron_table}
    echo "0 */1 * * * root [ -x ${script_folder}${cleanup_script} ] && nice -1 ${script_folder}${cleanup_script} ${cleanup_log} >> ${cleanup_log} 2>&1" >> ${cron_table}
    echo "15 1 * * * root [ -x -x /usr/bin/php5-cgi ] && nice -1 /usr/bin/php5-cgi -q ${cleanup_php} "cmd=del_cdrs" >> ${cleanup_log} 2>&1" >> ${cron_table}
fi
/etc/init.d/cron restart

# Install reporting replication script
replication_script="reporting_replication.sh"
if [ -f $replication_script ]; then
    chown ${app}:www-data ${replication_script}
    chmod 700 ${replication_script}
    mv ${replication_script} ${script_folder}
    replication_script=${script_folder}"/reporting_replication.sh"
else
    log_error "${replication_script} not available"
    on_error
    exit 1
fi

log_msg "reporting replication already compiled, fixing permissions"
mv reporting_replication.out /usr/local/bin/
chown ${app}:www-data /usr/local/bin/reporting_replication.out
chmod 700 /usr/local/bin/reporting_replication.out
replication_out="/usr/local/bin/reporting_replication.out"

# restart rsyslog to (re)create the log files
/etc/init.d/rsyslog restart
# do not change "Installation was successful", it will be matched!
log_msg "You may now configure the CDR interface at your PBX"
log_msg "The web server will be restarted a few seconds after the installation!"
log_msg "Installation was successful"
sleep 10s
php5-cgi /var/www/innovaphone/update_webserver_cfg.php

on_exit
exit 0
